//
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
/*  This version, HeikenAshi2v1 was modified by Mike Winfrey 3/15/2012
	A friend made a comment that he had a version of HeikenAshi on another charting package that allowed 
    an input to average the HeikenAshi close over a period.  That is what this modification does.  One line of code
	was modified from the original version that comes with NinjaTrader.  You will notice a parameter called period.
	A period of 1 which is the default is the same as the origina HeikenAshi.  Periods greater than 1 averages the 
	HeikenAshi Close over that number of bars. Oh yes, there is another modification.  The original HeikenAshi calculations
	do not correspond exactly with price.  I inserted a rounding function so the OHLC coincide exactly with price.	

	Please provide comments on any improvements or bugs.  My email address is mike.winfrey@gmail.com and 
	skype id is mikwinf.
*/
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
    /// </summary>
    [Description("HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.")]
    public class HeikenAshi2v1 : Indicator
    {
        #region Variables

        private Color           barColorDown         = Color.Red;
        private Color           barColorUp           = Color.Lime;
        private SolidBrush      brushDown            = null;
        private SolidBrush      brushUp              = null;
        private Color           savePenColor         = Color.Transparent;
        private Color           savePen2Color        = Color.Transparent;
        private Color           saveDownColor        = Color.Empty;
        private Color           saveUpColor          = Color.Empty;
        private Color           shadowColorUp        = Color.Lime;
        private Color           shadowColorDown      = Color.Red;
        private Color           shadowColorDoji      = Color.Gold;
      	private Pen             shadowPenUp          = null;
        private Pen				shadowPenDown		 = null;
        private Pen				shadowPenDoji		 = null;
		private int             shadowWidth          = 1;

		private int				period				 = 1;
		private bool			applytoall			 = false;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Gray, PlotStyle.Line, "HAOpen"));
            Add(new Plot(Color.Gray, PlotStyle.Line, "HAHigh"));
            Add(new Plot(Color.Gray, PlotStyle.Line, "HALow"));
            Add(new Plot(Color.Gray, PlotStyle.Line, "HAClose"));
            PaintPriceMarkers   = false;
            CalculateOnBarClose = false;
            PlotsConfigurable   = false;
            Overlay             = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {

            if (CurrentBar == 0)
            {
                HAOpen.Set(RoundPrice(Open[0]));
                HAHigh.Set(RoundPrice(High[0]));
                HALow.Set(RoundPrice(Low[0]));
                HAClose.Set(RoundPrice(Close[0]));
                return;
            }

            HAClose.Set(RoundPrice((Open[0] + High[0] + Low[0] + Close[0]) * 0.25)); // Calculate the close
            HAClose.Set(RoundPrice(SMA(HAClose,period)[0])); // Recalculates the HAClose by averaging the previous x HACloses
			
            HAOpen.Set(RoundPrice((HAOpen[1] + HAClose[1]) * 0.5)); // Calculate the open
			if (applytoall) HAOpen.Set(RoundPrice(SMA(HAOpen,period)[0]));
			
            HAHigh.Set(RoundPrice(Math.Max(High[0], HAOpen[0]))); // Calculate the high
			if (applytoall) HAHigh.Set(RoundPrice(Math.Max(High[0], SMA(HAHigh,period)[0])));
			
            HALow.Set(RoundPrice(Math.Min(Low[0], HAOpen[0]))); // Calculate the low
			if (applytoall) HALow.Set(RoundPrice(Math.Min(Low[0], SMA(HALow,period)[0])));
        }

        private string FormatPrice(double value)
        {
            return Bars.Instrument.MasterInstrument.FormatPrice(value);
        }

        private double RoundPrice(double value)
        {
            return Bars.Instrument.MasterInstrument.Round2TickSize(value);
        }
		
        #region Properties

        [Description("Apply period to all values")]
        [Category("Parameters")]
        [Gui.Design.DisplayNameAttribute("ApplyToAll")]
        public bool ApplyToAll
        {
            get { return applytoall; }
            set { applytoall = value; }
        }
        [Description("Period")]
        [Category("Parameters")]
        [Gui.Design.DisplayNameAttribute("Period")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(value, 1); }
        }
        [Browsable(false)]
        [XmlIgnore]
        public DataSeries HAOpen
        {
            get { return Values[0]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public DataSeries HAHigh
        {
            get { return Values[1]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public DataSeries HALow
        {
            get { return Values[2]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public DataSeries HAClose
        {
            get { return Values[3]; }
        }

        [XmlIgnore]
        [Description("Color of down bars.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("Candle Color Down")]
        public Color BarColorDown
        {
            get { return barColorDown; }
            set { barColorDown = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string BarColorDownSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(barColorDown); }
            set { barColorDown = Gui.Design.SerializableColor.FromString(value); }
        }

        /// <summary>
        /// </summary>
        [XmlIgnore]
        [Description("Color of up bars.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("Candle Color Up")]
        public Color BarColorUp
        {
            get { return barColorUp; }
            set { barColorUp = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string BarColorUpSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(barColorUp); }
            set { barColorUp = Gui.Design.SerializableColor.FromString(value); }
        }

        /// <summary>
        /// </summary>
        [XmlIgnore]
        [Description("Color of Shadow Line for UpClose.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("Shadow Color Up")]
        public Color ShadowColorUp
        {
            get { return shadowColorUp; }
            set { shadowColorUp = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string ShadowColorUpSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(shadowColorUp); }
            set { shadowColorUp = Gui.Design.SerializableColor.FromString(value); }
        }

        /// <summary>
        /// </summary>
        [XmlIgnore]
        [Description("Color of Shadow Line for DownClose.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("Shadow Color Down")]
        public Color ShadowColorDown
        {
            get { return shadowColorDown; }
            set { shadowColorDown = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string ShadowColorDownSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(shadowColorDown); }
            set { shadowColorDown = Gui.Design.SerializableColor.FromString(value); }
        }

         /// <summary>
        /// </summary>
        [XmlIgnore]
        [Description("Color of Doji Shadow Line.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("Shadow Color Doji")]
        public Color ShadowColorDoji
        {
            get { return shadowColorDoji; }
            set { shadowColorDoji = value; }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string ShadowColorDojiSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(shadowColorDoji); }
            set { shadowColorDoji = Gui.Design.SerializableColor.FromString(value); }
        }

       /// <summary>
        /// </summary>
        [Description("Width of shadow line.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("Shadow width")]
        public int ShadowWidth
        {
            get { return shadowWidth; }
            set { shadowWidth = Math.Max(value, 1); }
        }

        #endregion

        #region Miscellaneous

        protected override void OnStartUp()
        {
            if (ChartControl == null || Bars == null)
                return;
            savePenColor                        = Bars.BarsData.ChartStyle.Pen.Color;
            savePen2Color                       = Bars.BarsData.ChartStyle.Pen2.Color;
            saveUpColor                         = Bars.BarsData.ChartStyle.UpColor;
            saveDownColor                       = Bars.BarsData.ChartStyle.DownColor;
           	Bars.BarsData.ChartStyle.Pen.Color  = Color.Empty;
            Bars.BarsData.ChartStyle.Pen2.Color = Color.Empty;
            Bars.BarsData.ChartStyle.DownColor  = Color.Empty;
            Bars.BarsData.ChartStyle.UpColor    = Color.Empty;
            brushUp                             = new SolidBrush(barColorUp);
            brushDown                           = new SolidBrush(barColorDown);
           	shadowPenUp                         = new Pen(shadowColorUp, shadowWidth);
            shadowPenDown                       = new Pen(shadowColorDown, shadowWidth);
            shadowPenDoji                       = new Pen(shadowColorDoji, shadowWidth);
       }

        protected override void OnTermination()
        {
            if (ChartControl != null && Bars != null)
            {
                Bars.BarsData.ChartStyle.Pen.Color = savePenColor;
                Bars.BarsData.ChartStyle.Pen2.Color = savePen2Color;
                Bars.BarsData.ChartStyle.UpColor = saveUpColor;
                Bars.BarsData.ChartStyle.DownColor = saveDownColor;
            }

            if (brushUp != null) brushUp.Dispose();
            if (brushDown != null) brushDown.Dispose();
           	if (shadowPenUp != null) shadowPenUp.Dispose();
            if (shadowPenDown != null) shadowPenDown.Dispose();
            if (shadowPenDoji != null) shadowPenDoji.Dispose();
       }

        public override void GetMinMaxValues(ChartControl chartControl, ref double min, ref double max)
        {
            if (Bars == null || ChartControl == null)
                return;

            for (int idx = FirstBarIndexPainted; idx <= LastBarIndexPainted; idx++)
            {
                double tmpHigh = HAHigh.Get(idx);
                double tmpLow = HALow.Get(idx);

                if (tmpHigh != 0 && tmpHigh > max)
                    max = tmpHigh;
                if (tmpLow != 0 && tmpLow < min)
                    min = tmpLow;
            }
        }

        public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
        {
            if (Bars == null || ChartControl == null)
                return;

            int barPaintWidth = Math.Max(3, 1 + 2 * ((int)Bars.BarsData.ChartStyle.BarWidth - 1) + 2 * shadowWidth);

            for (int idx = FirstBarIndexPainted; idx <= LastBarIndexPainted; idx++)
            {
                if (idx - Displacement < 0 || idx - Displacement >= BarsArray[0].Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
                    continue;
                double valH = HAHigh.Get(idx);
                double valL = HALow.Get(idx);
                double valC = HAClose.Get(idx);
                double valO = HAOpen.Get(idx);
                int x = ChartControl.GetXByBarIdx(BarsArray[0], idx);
                int y1 = ChartControl.GetYByValue(this, valO);
                int y2 = ChartControl.GetYByValue(this, valH);
                int y3 = ChartControl.GetYByValue(this, valL);
                int y4 = ChartControl.GetYByValue(this, valC);

                
                if (y4 == y1)
				{
               		graphics.DrawLine(shadowPenDoji, x, y2, x, y3);
                    graphics.DrawLine(shadowPenDoji, x - barPaintWidth / 2, y1, x + barPaintWidth / 2, y1);
				}
                else
                {
                    if (y4 > y1)
					{
                		graphics.DrawLine(shadowPenDown, x, y2, x, y3);
						graphics.FillRectangle(brushDown, x - barPaintWidth / 2, y1, barPaintWidth, y4 - y1);
                   		graphics.DrawRectangle(shadowPenDown, (x - barPaintWidth / 2) + (shadowPenUp.Width / 2), Math.Min(y4, y1), barPaintWidth - shadowPenUp.Width, Math.Abs(y4 - y1));
                    }
					else
					{
                		graphics.DrawLine(shadowPenUp, x, y2, x, y3);
                        graphics.FillRectangle(brushUp, x - barPaintWidth / 2, y4, barPaintWidth, y1 - y4);
                   		graphics.DrawRectangle(shadowPenUp, (x - barPaintWidth / 2) + (shadowPenDown.Width / 2), Math.Min(y4, y1), barPaintWidth - shadowPenDown.Width, Math.Abs(y4 - y1));
					}
				}
            }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private HeikenAshi2v1[] cacheHeikenAshi2v1 = null;

        private static HeikenAshi2v1 checkHeikenAshi2v1 = new HeikenAshi2v1();

        /// <summary>
        /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public HeikenAshi2v1 HeikenAshi2v1(bool applyToAll, int period)
        {
            return HeikenAshi2v1(Input, applyToAll, period);
        }

        /// <summary>
        /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public HeikenAshi2v1 HeikenAshi2v1(Data.IDataSeries input, bool applyToAll, int period)
        {
            if (cacheHeikenAshi2v1 != null)
                for (int idx = 0; idx < cacheHeikenAshi2v1.Length; idx++)
                    if (cacheHeikenAshi2v1[idx].ApplyToAll == applyToAll && cacheHeikenAshi2v1[idx].Period == period && cacheHeikenAshi2v1[idx].EqualsInput(input))
                        return cacheHeikenAshi2v1[idx];

            lock (checkHeikenAshi2v1)
            {
                checkHeikenAshi2v1.ApplyToAll = applyToAll;
                applyToAll = checkHeikenAshi2v1.ApplyToAll;
                checkHeikenAshi2v1.Period = period;
                period = checkHeikenAshi2v1.Period;

                if (cacheHeikenAshi2v1 != null)
                    for (int idx = 0; idx < cacheHeikenAshi2v1.Length; idx++)
                        if (cacheHeikenAshi2v1[idx].ApplyToAll == applyToAll && cacheHeikenAshi2v1[idx].Period == period && cacheHeikenAshi2v1[idx].EqualsInput(input))
                            return cacheHeikenAshi2v1[idx];

                HeikenAshi2v1 indicator = new HeikenAshi2v1();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ApplyToAll = applyToAll;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                HeikenAshi2v1[] tmp = new HeikenAshi2v1[cacheHeikenAshi2v1 == null ? 1 : cacheHeikenAshi2v1.Length + 1];
                if (cacheHeikenAshi2v1 != null)
                    cacheHeikenAshi2v1.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheHeikenAshi2v1 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HeikenAshi2v1 HeikenAshi2v1(bool applyToAll, int period)
        {
            return _indicator.HeikenAshi2v1(Input, applyToAll, period);
        }

        /// <summary>
        /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public Indicator.HeikenAshi2v1 HeikenAshi2v1(Data.IDataSeries input, bool applyToAll, int period)
        {
            return _indicator.HeikenAshi2v1(input, applyToAll, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.HeikenAshi2v1 HeikenAshi2v1(bool applyToAll, int period)
        {
            return _indicator.HeikenAshi2v1(Input, applyToAll, period);
        }

        /// <summary>
        /// HeikenAshi2 technique discussed in the article 'Using Heiken-Ashi Technique' in February 2004 issue of TASC magazine.
        /// </summary>
        /// <returns></returns>
        public Indicator.HeikenAshi2v1 HeikenAshi2v1(Data.IDataSeries input, bool applyToAll, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.HeikenAshi2v1(input, applyToAll, period);
        }
    }
}
#endregion
